/**==========================================================================

	PC-8001 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2010 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	External assembler functions for IBM-PC with VC++ 5.0
	last modified  Apr,4,1998.

;==========================================================================*/
#include "pc-8001.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
export BYTE __cdecl _n80_ioread(USHORT port)
{
	USHORT portno = port & 0x0f;
	switch ((port & 0xf0)) {
	case 0x00:
			if (portno < 12)
				return n80device.keys[portno];
			break;

	case 0x20:
			if (portno == 0)
				return n80device.usart.read();
			else if (portno == 1)
				return n80device.usart.status();
			else
				break;

	case 0x30:
			if (portno == 0)		// SW-1 (30H)
				return n80device.param.dipsw1.SW1_N_ROM == 1 ? 0xd3 : 0xd2;	//0:N88, 1:N80
			else if (portno == 1)	// SW-2 (31H)
				return 0xeb;
			break;

	case 0x40:	// printer always ready
			return (n80device.vrtc.poll(n80device.io._i40) & 0xfe);
			
	case 0x50:
			if (portno == 0xc)		// G-VRAM status (5CH)
				return n80device.io._i5C;
			break;
			
	case 0x60:
			if (portno == 8)
				return 0xff;
			break;
			
	case 0x70:
			if (portno == 0)
				return n80device.io._o70;
			else if (portno == 1) {
				if (n80device.io._o71 == 0)
					n80device.io._o71 = 0xff;
				return n80device.io._o71;
			}
			break;
			
	case 0xe0:
			if (portno == 4)
				return n80device.io._oE4;
			else if (portno == 6)
				return n80device.io._oE6;
			break;
	}
	return 0xff;
}

export void __cdecl _n80_iowrite(USHORT port, BYTE v)
{
	BYTE portno = port & 0x0f;
	switch ((port & 0xf0)) {
	case 0x00:
			n80device.hal.mode(portno, v);
			break;

	case 0x10:
			n80device.io._o10 = v;
			break;

	case 0x20:
			if ((portno & 0x01) == 0) {
				n80device.io._o20 = v;
				n80device.usart.write(v);
			} else {
				n80device.io._o21 = v;
				n80device.usart.cmd();
			}
			break;

	case 0x30:
			if (portno == 0) {
				n80device.io._o30 = v;
				n80device.crtc.mode(v);
				n80device.cmt.control();
			} else
				n80device.io._o31 = v;
			break;

	case 0x40:
			if (portno == 0) {
				n80device.io._o40 = v;
				n80device.calender.mode();
				if (v & _PO40_BEEP)
					n80device.snd.beep_on();
				else
					n80device.snd.beep_off();
			}
			break;

	case 0x50:
			//printf("OUT(%2x,%2x)\n", port & 0xff, v);
			if (portno == 0) {
				n80device.io._o50 = v;	// CRTC param (50H)
				n80device.crtc.data(v);
			} else if (portno == 1) {
				n80device.io._o51 = v;	// CRTC command (51H)
				n80device.crtc.cmd(v);
			} else if (portno == 2) {
				n80device.io._o52 = v;	// bg/boader color (52H)
			} else if (portno == 3) {
				n80device.io._o53 = v;	// T/G-VRAM visible (53H)
				if (v & 0x01)
					n80device.crtc._text_visible |= 1;
				else
					n80device.crtc._text_visible &= 0xfe;
			} else if (portno == 4) {	// color palette #0
				n80device.palette[0] = v & 0x7;
			} else if (portno == 5) {	// color palette #1
				n80device.palette[1] = v & 0x7;
			} else if (portno == 6) {	// color palette #2
				n80device.palette[2] = v & 0x7;
			} else if (portno == 7) {	// color palette #3
				n80device.palette[3] = v & 0x7;
			} else if (portno == 8) {	// color palette #4
				n80device.palette[4] = v & 0x7;
			} else if (portno == 9) {	// color palette #5
				n80device.palette[5] = v & 0x7;
			} else if (portno == 0xa) {	// color palette #6
				n80device.palette[6] = v & 0x7;
			} else if (portno == 0xb) {	// color palette #7
				n80device.palette[7] = v & 0x7;
			} else if (portno == 0xc) {
				n80device.io._i5C = 0xf9;	// GVRAM#0 selected
			} else if (portno == 0xd) {
				n80device.io._i5C = 0xfa;	// GVRAM#1 selected
			} else if (portno == 0xe) {
				n80device.io._i5C = 0xfc;	// GVRAM#2 selected
			} else if (portno == 0xf) {
				n80device.io._i5C = 0xf8;	// main RAM selected
			}
			break;
			
	case 0x60:
			switch (portno) {
			case 0:	n80device.dmac.ch_adr(0, v);	break;
			case 1:	n80device.dmac.ch_cnt(0, v);	break;
			case 2:	n80device.dmac.ch_adr(1, v);	break;
			case 3:	n80device.dmac.ch_cnt(1, v);	break;
			case 4:	n80device.dmac.ch_adr(2, v);	break;
			case 5:	n80device.dmac.ch_cnt(2, v);	break;
			case 6:	n80device.dmac.ch_adr(3, v);	break;
			case 7:	n80device.dmac.ch_cnt(3, v);	break;
			case 8:	n80device.dmac.cmd(v);			break;
			}
			break;
			
	case 0x70:
			if (portno == 0)
				n80device.io._o70 = v;	//TEXT WINDOW offset (70H)
			else if (portno == 1) {
				n80device.io._o71 = v;	//4th ROM SELECT (71H)
				n80device.io._i71 = v;
			} else if (portno == 8)
				++n80device.io._o70;	//TEXT WINDOW offset inc (78H)
			break;
	
	case 0xe0:
			if (portno == 4)
				n80device.io._oE4 = v;	// PB8214 interrupt level
			else if (portno == 6)
				n80device.io._oE6 = v;	// PB8214 interrupt mask
			break;

	case 0xf0:
			if (portno == 0) {			// fast cmt load
				if (v > 1) {
					int i;
					for (i = 0; i < v; ++i)
						n80device.cmt.fastload();
				} else
					n80device.cmt.fastload();
			}
			break;
	}
}
	
#ifdef __cplusplus
}	// extern "C"
#endif
