//
//  PB8214.c
//  N88X
//
//  Created by hiroshi on 2016/05/01.
//  Copyright © 2016年 H.Kanemoto. All rights reserved.
//
#include "pc-8001.h"
#ifdef _MSC_VER
#include <windows.h>
#include <crtdbg.h>
#endif
#ifdef __cplusplus
extern "C"
{
#endif
#define export
#include <stdio.h>
static int req1_count = 0, req1_accept = 0, req1_level = 0, req1_mask = 0, req1_overrun = 0;
static int req2_count = 0, req2_accept = 0, req2_level = 0, req2_mask = 0, req2_overrun = 0;
#ifdef _MSC_VER
static char buff[128];
#endif
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
static int interrupt_request(LPZ80CPU cpu, BYTE mask, BYTE lev) {
	//check interrupt disabled
	if ((n80device.io._oE6 & mask) == 0)
		return 3;
	//check interrupt enabled level
	if ((n80device.io._oE4 & 0x08) == 0) {
		BYTE level = n80device.io._oE4 & 0x07;
		if (level > lev)
			;
		else
			return 2;
	}
	return cpu->int_signal(cpu, lev);
}

// resend holding interrupt
static void intx_resend(LPZ80CPU cpu) {
	if (n80device.pic._holding == 0)
		return;
	if ((n80device.pic._holding & 1) == 1) {
		switch (interrupt_request(cpu, 0x04, 0)) {
			case 0:	// accepted
				n80device.pic._holding &= 0xfe;	break;
			case 1:	// DI
			case 2:	// Level
			case 3:	// Masked
				break;
		}
		return;
	}
	if ((n80device.pic._holding & 3) == 2) {
		switch (interrupt_request(cpu, 0x02, 1)) {
			case 0:	// accepted
				++req1_accept;
				n80device.pic._holding &= 0xfd;	break;
			case 1:	// DI
			case 2:	// Level
				break;
			case 3:	// Masked
				n80device.pic._holding &= 0xfd;	break;
		}
		return;
	}
	if ((n80device.pic._holding & 7) == 4) {
		switch (interrupt_request(cpu, 0x01, 2)) {
			case 0:	// accepted
				++req2_accept;
				n80device.pic._holding &= 0xfb;	break;
			case 1:	// DI
			case 2:	// Level
				break;
			case 3:	// Masked
				n80device.pic._holding &= 0xfb;	break;
		}
	}
}

// USART ready(rxd) interrupt
static void int0_request(LPZ80CPU cpu) {
	n80device.pic._irq_0 = 1;
	if (interrupt_request(cpu, 0x04, 0) != 0)
		n80device.pic._holding |= 1;
}

// VRTC vertical blanking interrupt
static void int1_request(LPZ80CPU cpu) {
	++req1_count;
	if ((n80device.pic._holding & 2) == 2) {
		++req1_overrun;
		return;
	}
	switch (interrupt_request(cpu, 0x02, 1)) {
		case 0:
			++req1_accept;
			break;
		case 1:
			n80device.pic._holding |= 2;
			break;
		case 2:	//level
			++req1_level;
			break;
		case 3: //masked
			++req1_mask;
			break;
	}
#if _DEBUG
	if (req1_count > 120) {
		double percent = ((double)req1_accept) / ((double)req1_count) * 100;
        double percent2 = ((double)req1_level) / ((double)req1_count) * 100;
		double percent3 = ((double)req1_mask) / ((double)req1_count) * 100;
		double percent4 = ((double)req1_overrun) / ((double)req1_count) * 100;
#if _MSC_VER
		sprintf(buff, "VRTC vertical blinking %.1f%%(level:%.1f%%, mask:%.1f%%, over:%.1f%%)\n", percent, percent2, percent3, percent4);
		_RPT0(_CRT_WARN, buff);
#else
		printf("VRTC vertical blinking %.1f%%(level:%.1f%%, mask:%.1f%%, over:%.1f%%)\n", percent, percent2, percent3, percent4);
#endif
		req1_count = 0;
		req1_accept = 0;
		req1_level = 0;
		req1_mask = 0;
		req1_overrun = 0;
	}
#endif
}

// RTC(1/600sec) interrupt
static void int2_request(LPZ80CPU cpu) {
 	++req2_count;
    if ((n80device.pic._holding & 4) == 4) {
        ++req2_overrun;
    } else {
        switch (interrupt_request(cpu, 0x01, 2)) {
            case 0:
                ++req2_accept;
                break;
            case 1:
                n80device.pic._holding |= 4;
                break;
            case 2: //level
                ++req2_level;
                break;
            case 3:	//masked
                ++req2_mask;
                break;
        }
    }
#if _DEBUG
	if (req2_count > 1200) {
		double percent = ((double)req2_accept) / ((double)req2_count) * 100;
		double percent2 = ((double)req2_level) / ((double)req2_count) * 100;
        double percent3 = ((double)req2_mask) / ((double)req2_count) * 100;
		double percent4 = ((double)req2_overrun) / ((double)req2_count) * 100;
#if _MSC_VER
		sprintf(buff, "RTC clock interrupt %.1f%%(level:%.1f%%, mask:%.1f%%, over:%.1f%%)\n", percent, percent2, percent3, percent4);
		_RPT0(_CRT_WARN, buff);
#else
		printf("RTC clock interrupt %.1f%%(level:%.1f%%, mask:%.1f%%, over:%.1f%%)\n", percent, percent2, percent3, percent4);
#endif
		req2_count = 0;
		req2_accept = 0;
		req2_level = 0;
		req2_mask = 0;
		req2_overrun = 0;
	}
#endif
}
	
static void _n80_reset_pic(void) {
	n80device.pic._holding = 0;
	n80device.pic._result = 0;
	n80device.pic._irq_0 = 0;
}

export void _n80_init_pic(void) {
	n80device.pic.reset = _n80_reset_pic;
	n80device.pic.resend = intx_resend;
	n80device.pic.int0 = int0_request;
	n80device.pic.int1 = int1_request;
	n80device.pic.int2 = int2_request;
	_n80_reset_pic();
}
	
#ifdef __cplusplus
}	// extern "C"
#endif
